function [ap rec prec tab] = evaluation_fichier(resfilename,foldID,classID,resolution,display,register_falsepos_and_weaktruepos)
% evaluate one file
% resfilename contains the results with imgID X Y Score
% foldID is the fold ID (1 to 10)
% classID is the class ID (1: car, 2:trucks, 4: tractors, 5: camping cars, 9: vans, 10: others, 11: pickup, 23: boats , 201: Small Land Vehicles, 301: Large land Vehicles
% resolution is either 512 or 1024, depending the set tested
% display set to true will show the images and the corresponding results, as well as the two strongest score of the current image
% register_falsepos_and_weaktruepos set to true will save strong false positive and weak true positive
% written by S. Razakarivony
% srazakarivony@gmail.com
% for Sagem DS and GREYC

ap = -1;
rec = -1;
prec = -1;
tab = zeros(1,5);


try
	if (resolution==1024)
		AnnotationsReference = load('../Annotations1024/annotation1024.txt');
		IMGSIZE = 1024;
	else
		if (resolution==512)
			AnnotationsReference = load('../Annotations512/annotation512.txt');
			IMGSIZE = 512;
		else
			fprintf(1,'Please enter a resolution of the possible sets, 512 or 1024');
			return;
		end
	end

	catch exceptionFile
		fprintf('%s\n', exceptionFile.message);
		fprintf('Invalide annotation file. Find annotation1024.txt and annotation512.txt and move them to ../Annotations1024/ or ../Annotations512/ respectivly\n');	
	return;
end

try
	if (resolution==1024)
		load('../Annotations1024/fold.mat');
	end
	if (resolution==512)
		load('../Annotations512/fold.mat');
	end
catch exceptionFile
	fprintf('%s\n', exceptionFile.message);
	fprintf('Invalide fold file. Find fold.mat and move it to ../Annotations1024/ and ../Annotations512/\n');	
	return;
end

Reference = [];
%take only position corresponding to the classID
A = AnnotationsReference;%to read easily
for indim=1:size(fold,2)
	switch classID
		case 201
			Reference = [Reference ; A(((A(:,end-2)~=5)&(A(:,end-2)~=2)&(A(:,end-2)~=23)&(A(:,end-2)~=31))&(A(:,1)==fold(foldID,indim)),:)];
		case 301
			Reference = [Reference ; A(((A(:,end-2)==5)|(A(:,end-2)==2))&(A(:,1)==fold(foldID,indim)),:)];
		otherwise
			Reference = [Reference ; A((A(:,end-2)==classID)&(A(:,1)==fold(foldID,indim)),:)];
	end
end

%count positive for this classID
NbofPositiveExamples = size(Reference,1);
fprintf(1,'There is %d positive in this fold\n',NbofPositiveExamples);

%load resultats
try
	Resultats = load(resfilename);
catch exceptionFile
	fprintf('%s\n', exceptionFile.message);
	fprintf('Invalide result file name\n');	
	return;
end

%some index are initialized
index_threshold = 1;
index_reference = 1;
index_image_ref = 1;
index_neg = 1;
index_pos = 1;
%count the double detection
doubledetect=0;


%list of image names of the results
try
	list_of_num = unique(Resultats(:,1));
catch exceptionResult
	fprintf(1,'%s\n', exceptionResult.message);
	fprintf(1,'Your result file seems empty or something\n');	
	return;
end
fprintf(1,'You gave results for %d images\n',length(list_of_num));

%to keep false positive and weak positive
if (register_falsepos_and_weaktruepos)
	window_to_keep = zeros(min(size(Resultats,1),10000),48*48+1);
end

inddes = 1;
indF = 1;
forgottenOne = [];

for i =1:length(list_of_num)
	if (mod(i,50)==0)
		fprintf('compute image %d (num = %08d)\n',i,list_of_num(i));
	end
	
	ThisImageRef = Reference(Reference(:,1)==list_of_num(i),:);
	ThisImageResults = Resultats(Resultats(:,1)==list_of_num(i),:);
	
	if (~isempty(ThisImageResults))
		%sort the results
		[sortedvalue sortedind] = sort(ThisImageResults(:,4),'descend');
		ThisImageResults = ThisImageResults(sortedind,:);
		ThisClassInformations = ThisImageRef(:,2:end);
		PositionsResults = ThisImageResults(:,2:3);

		%initialisation of targets
		for index_cible=1:size(ThisClassInformations,1)
			alreadyTaken(index_cible)=false;
		end
		
		if (display)
			try
			if (resolution==1024)
				I = imread(sprintf('../Vehicules1024/%08d_co.png',ThisImageResults(1,1)));
			end
			if (resolution==512)
				I = imread(sprintf('../Vehicules512/%08d_co.png',ThisImageResults(1,1)));
			end
			catch exceptionFile
				fprintf('%s\n', exceptionFile.message);
				fprintf('Invalide fold file. Find where your images files are and move them to ../Vehicules/\n');
				return;
			end
			subplot(1,2,1)
			hold off
			imagesc(I)
			axis equal
			axis off
			hold on
			plot(ThisClassInformations(:,1),ThisClassInformations(:,2),'+g')
			hold on
			plot(PositionsResults(:,1),PositionsResults(:,2),'+')
			
			%draw 2 best results
			if (size(PositionsResults,1)>2)
				plot(PositionsResults(1:2,1),PositionsResults(1:2,2),'oy')
				subplot(2,2,2)
				if ((PositionsResults(1,2)>23)&&(PositionsResults(1,1)>23))
					if ((PositionsResults(1,2)<IMGSIZE-24)&&(PositionsResults(1,1)<IMGSIZE-24))
						Imagette = I(ceil(PositionsResults(1,2)-23):floor(PositionsResults(1,2)+23),ceil(PositionsResults(1,1)-23):floor(PositionsResults(1,1)+23),:);
						imagesc(Imagette)
					end
				end
				axis equal
				axis off
				subplot(2,2,4)
				if ((PositionsResults(2,2)>24)&&(PositionsResults(2,1)>24))
					if ((PositionsResults(2,2)<IMGSIZE-25)&&(PositionsResults(2,1)<IMGSIZE-25))
						Imagette = I(round(PositionsResults(2,2)-23:PositionsResults(2,2)+23),round(PositionsResults(2,1)-23:PositionsResults(2,1)+23),:);
						imagesc(Imagette)
					end
				end
				axis equal
				axis off
			end
			subplot(1,2,1)
		end

		%take all reference and build ellipses from them
		ellipses_struct = [];
		for v=1:size(ThisClassInformations,1)
			orientationVehicle = ThisClassInformations(v,3);
			xRef = ThisClassInformations(v,1);
			yRef = ThisClassInformations(v,2);	
			X = ThisClassInformations(v,4:7);
			Y = ThisClassInformations(v,8:11);
			l(1) = sqrt((X(1)-X(2)+1)^2+(Y(1)-Y(2)+1)^2);
			l(2) = sqrt((X(2)-X(3)+1)^2+(Y(2)-Y(3)+1)^2);
			l(3) = sqrt((X(3)-X(4)+1)^2+(Y(3)-Y(4)+1)^2);
			l(4) = sqrt((X(4)-X(1)+1)^2+(Y(4)-Y(1)+1)^2);
			[ltri,ktri] = sort(l,'descend');
			longueurVehicle = mean([ltri(1) ltri(2)]);
			largeurVehicle = mean([ltri(3) ltri(4)]);
			ellipse(longueurVehicle/2,largeurVehicle/2,orientationVehicle,xRef,yRef,'r',600);
			ellipses_struct = [ellipses_struct;longueurVehicle/2,largeurVehicle/2,orientationVehicle,xRef,yRef];
		end

		
		if (register_falsepos_and_weaktruepos)
			Im = zeros(size(I,1)+48,size(I,2)+48);
			Im(25:end-24,25:end-24)=(double(I(:,:,1))+double(I(:,:,2))+double(I(:,:,3)))/3.0;
		end
		
		%for all results, check if it is within the ellipse
		for u=1:size(PositionsResults,1)
			detect=false;
			xres = PositionsResults(round(u),1);
			yres = PositionsResults(round(u),2);
			if (register_falsepos_and_weaktruepos)
				if inddes<=size(window_to_keep,1)
					if ((yres>24)&&(xres>24))
						if ((yres<IMGSIZE-25)&&(xres<IMGSIZE-25))
							tmp = double(reshape(Im(round(yres:yres+47),round(xres:xres+47)),1,48*48));
							window_to_keep(inddes,:) = [tmp-1];
							indice_desc(inddes) = list_of_num(i);
						end
					end
				end
			end
			ok = false;
			ValuesNegatives(index_neg) = ThisImageResults(u,4);
			index_neg = index_neg+1;
			for index_ok = 1:size(ellipses_struct,1)
				X = ellipses_struct(index_ok,1);
				Y = ellipses_struct(index_ok,2);
				a = ellipses_struct(index_ok,4);
				b = ellipses_struct(index_ok,5);
				alpha = -ellipses_struct(index_ok,3);
				xx = xres-a;
				yy = yres-b;
				x = cos(alpha)*xx-sin(alpha)*yy;
				y = sin(alpha)*xx+cos(alpha)*yy;
				if ((x/X)^2+(y/Y)^2)<=1.0
					if (display)
						ellipse(ellipses_struct(index_ok,1),ellipses_struct(index_ok,2),ellipses_struct(index_ok,3),ellipses_struct(index_ok,4),ellipses_struct(index_ok,5),'g',600);
						plot(xres,yres,'og')
					end
					detect = true;
					ind = index_ok;
				end
			end
			if (detect)
				ok = true;
				v = ind;
				if (alreadyTaken(v)==false)
					if (display)
						plot(xres,yres,'+g')
					end
					ValuesPositives(index_pos) = ThisImageResults(u,4);
					if (register_falsepos_and_weaktruepos)
						if inddes<=size(window_to_keep,1)
							window_to_keep(inddes,end)=1;
						end
					end
					index_pos = index_pos+1;
					alreadyTaken(v)=true;
				else
					if (register_falsepos_and_weaktruepos)
						if inddes<=size(window_to_keep,1)
							window_to_keep(inddes,end)=0;
						end
					end
				end
			end
			if ok
				index_neg = index_neg-1;
			end
			inddes = inddes+1;
		end
		for indexCible=1:size(ThisClassInformations,1)
			if(alreadyTaken(indexCible)==false)
				xRef = floor(ThisClassInformations(indexCible,1));
				yRef = floor(ThisClassInformations(indexCible,2));
				if (register_falsepos_and_weaktruepos)
					if ((xRef<IMGSIZE)&&(yRef<IMGSIZE))&&((xRef>0)&&(yRef>0))
						forgottenOne(indF,:) = double(reshape(Im(yRef:yRef+47,xRef:xRef+47),1,48*48));
					end
				end
				indF = indF+1;
			end
		end
		
		if (display)
			drawnow
		end
	end
end

%compute the precision/Recall
index_threshold=1;
thresholds = sort(ValuesPositives,'descend')	;
for threshold = thresholds
	truePos = sum(ValuesPositives>=threshold);
	falsePos = sum(ValuesNegatives>=threshold);
	rec(index_threshold) = truePos/NbofPositiveExamples;
	prec(index_threshold) = truePos/(truePos+falsePos);	
	frac(index_threshold) = falsePos/size(fold,2);
	index_threshold = index_threshold +1;
end
meta_index = 1;

%compute operating points, with an interpolation
for index=1:size(frac,2)-1
	if (frac(index)<10)
		tab(5) = ((10-frac(index))/(frac(index+1)-frac(index)))*rec(index+1)+((frac(index+1)-10)/(frac(index+1)-frac(index)))*rec(index);
		if frac(index+1)<10
			tab(5) = rec(index+1);
		end
		meta_index= index;
	end
	if (frac(index)<1.0)
		tab(4) = (((1-frac(index))/(frac(index+1)-frac(index)))*rec(index+1)+((frac(index+1)-1)/(frac(index+1)-frac(index)))*rec(index));
		if frac(index+1)<1
			tab(4) = rec(index+1);
		end
	end
	if (frac(index)<.1)
		tab(3) = (( (.1-frac(index))/(frac(index+1)-frac(index)))*rec(index+1)+( (frac(index+1)-.1)/(frac(index+1)-frac(index)))*rec(index));
		if frac(index+1)<.1
			tab(3) = rec(index+1);
		end
	end
	if (frac(index)<.01)
		tab(2) = (((.01-frac(index))/(frac(index+1)-frac(index)))*rec(index+1)+((frac(index+1)-.01)/(frac(index+1)-frac(index)))*rec(index));
		if frac(index+1)<.01
			tab(2) = rec(index+1);
		end
	end
	if (frac(index)<.001)
		tab(1) = (((.001-frac(index))/(frac(index+1)-frac(index)))*rec(index+1)+((frac(index+1)-.001)/(frac(index+1)-frac(index)))*rec(index));
		if frac(index+1)<.001
			tab(1) = rec(index+1);
		end
	end
end

%compute ap
ap=0;
for t=0:0.1:1
    p=max(prec(rec>=t));
    if isempty(p)
        p=0;
    end
    ap=ap+p/11;
end

if (display)
	subplot(2,2,4)
	plot(rec,prec,'--+')
	axis([0 1 0 1])
	subplot(2,2,2)
	semilogx(frac,rec,'--+')
	V = axis();
	axis([V(1) V(2) 0 1])
end

fprintf(1,'ap %f\n',ap);
for i=1:5
	fprintf('%f ',tab(i))
end
fprintf('\n')

if (register_falsepos_and_weaktruepos)
	
	descP = window_to_keep(window_to_keep(:,end)==1,1:end-1);
	indices_descP = indice_desc(window_to_keep(:,end)==1);
	indices_descN = indice_desc(window_to_keep(:,end)==-1);
	descN = window_to_keep(window_to_keep(:,end)==-1,1:end-1);
	size_descP = size(descP);
	size_descN = size(descN);
	
	i=1;
	ValuesPositives = ValuesPositives(1:size(descP,1));
	ValuesNegatives = ValuesNegatives(1:size(descN,1));
	descPG = [];
	descNG = [];

	for indx =1:size(ValuesPositives,2)
		if ValuesPositives(indx)<thresholds(meta_index)
			descPG(i,:) = descP(indx,:);
			inddescPG(i)= indices_descP(indx);
			scorePG(i) = ValuesPositives(indx);
			i = i+1;
		end
	end

	size_descPG = size(descPG);
	scorePG = [scorePG -2.0*ones(1,size(forgottenOne,1))];
	descPG = [descPG;forgottenOne];
	i = 1;
	[ValuesNegatives idx] = sort(ValuesNegatives,'descend');
	descN = descN(idx,:);
	for indx =1:size(ValuesNegatives,2)
		if ValuesNegatives(indx)>thresholds(meta_index)
			descNG(i,:) = descN(indx,:);
			inddescNG(i)= indices_descN(indx);
			scoreNG(i) = ValuesNegatives(indx);
			i = i+1;
		end
	end

	a = size(descPG,1);
	r = min(floor(sqrt(a)),5);
	for indx = 1:r*r
		subplot(r,r,indx)
		imagesc(reshape(descPG(indx,1:end),48,48));
		colormap gray
		axis equal
	end

	a = size(descNG,1);
	r = min(floor(sqrt(a)),5);
	for indx = 1:r*r
		subplot(r,r,indx)
		imagesc(reshape(descNG(indx,1:end),48,48));
		colormap gray
		axis equal
		axis off
	end

	save(sprintf('classID_%d_fold_%d_descripteursFP.mat',classID,foldID),'descNG');
	save(sprintf('classID_%d_fold_%d_score.mat',classID,foldID),'scoreNG');
	save(sprintf('classID_%d_fold_%d_indiceFP.mat',classID,foldID),'inddescNG');
	save(sprintf('classID_%d_fold_%d_descripteursFN.mat',classID,foldID),'descPG');
	save(sprintf('classID_%d_fold_%d_scoreFN.mat',classID,foldID),'scorePG');
	save(sprintf('classID_%d_fold_%d_indiceFN.mat',classID,foldID),'inddescPG');
end
end
